class FIRFilter {
  late int length;
  late List<double> delayLine;
  late List<double> impulseResponse;
  late List<int> delayLine_int;
  late List<int> impulseResponse_int;
  int count = 0;

  FIRFilter.initWithDouble(List<double> coefs) {
    length = coefs.length;
    impulseResponse = coefs;
    delayLine = List.filled(length, 0);
    delayLine_int = List.filled(length, 0);
  }

  FIRFilter.initWithInt(List<int> coefs_int) {
    length = coefs_int.length;
    impulseResponse_int = coefs_int;
    delayLine = List.filled(length, 0);
    delayLine_int = List.filled(length, 0);
  }

  int applyWithInt(int inputSample) {
    delayLine_int[count] = inputSample;
    int result = 0;
    int index = count;
    for (int i = 0; i < length; i++) {
      result += (impulseResponse_int[i]) * delayLine_int[index--];
      if (index < 0) index = length - 1;
    }
    if (++count >= length) count = 0;
    return (result >> 16);
  }

  double applyWithDouble(double inputSample) {
    delayLine[count] = inputSample;
    double result = 0.0;
    int index = count;
    for (int i = 0; i < length; i++) {
      result += impulseResponse[i] * delayLine[index--];
      if (index < 0) index = length - 1;
    }
    if (++count >= length) count = 0;
    return result;
  }

  void clearValue() {
    for (int i = 0; i < length; i++) {
      delayLine[i] = 0;
    }
  }
}
